import {GeolocationCoordinates} from "../GeolocationCoordinates.js";
import {GeolocationShape} from "../GeolocationShape.js";

/**
 * @typedef {Object} GeolocationDataLayerSettings
 *
 * @property {String} import_path
 * @property {Object} settings
 * @property {Object} features
 * @property {String[]} scripts
 * @property {String[]} stylesheets
 * @property {GeolocationMapMarker[]} markers
 * @property {GeolocationShape[]} shapes
 */

export default class GeolocationDataLayer {

  /**
   * @param {GeolocationMapBase} map
   * @param {String} id
   * @param {GeolocationDataLayerSettings} settings
   */
  constructor(map, id, settings) {
    this.map = map;
    this.settings = settings;
    this.features = [];
    this.markers = [];
    this.shapes = [];
    this.id = id;
  }

  /**
   * @param {GeolocationLayerFeatureSettings} layerFeatureSettings
   * @return {Promise<GeolocationLayerFeature>|null}
   */
  loadFeature(layerFeatureSettings) {
    if (!layerFeatureSettings.import_path) {
      return null;
    }

    let scripts = layerFeatureSettings.scripts || [];
    let scriptLoads = [];
    for (const script of scripts) {
      scriptLoads.push(Drupal.geolocation.addScript(script));
    }

    let stylesheets = layerFeatureSettings.stylesheets || [];
    let stylesheetLoads = [];
    for (const stylesheet of stylesheets) {
      stylesheetLoads.push(Drupal.geolocation.addStylesheet(stylesheet));
    }

    return Promise.all(scriptLoads)
      .then(() => {
        return Promise.all(stylesheetLoads);
      })
      .then(() => {
        return import(layerFeatureSettings.import_path);
      })
      .then((featureImport) => {
        try {
          const feature = new featureImport.default(layerFeatureSettings.settings, this);
          this.features.push(feature);

          return feature;
        } catch (e) {
          console.error(e.toString(), "Loading feature failed");
          return null;
        }
      })
      .catch((error) => {
        console.error(error.toString(), "Loading '" + layerFeatureSettings.import_path + "' failed");
      });
  }

  async loadFeatures() {
    let featureImports = [];

    for (const featureName in this.settings.features) {
      const featurePromise = this.loadFeature(this.settings.features[featureName]);

      if (featurePromise) {
        featureImports.push(featurePromise);
      }
    }

    return Promise.all(featureImports).then(() => {
      return this;
    });
  }

  /**
   * @param {String} selector
   */
  async loadMarkers(selector = "") {
    if (!selector) {
      selector = "#" + this.id + ".geolocation-map-layer .geolocation-location";
    }

    this.map.wrapper.querySelectorAll(selector).forEach((location) => {
      let marker = this.map.createMarker(new GeolocationCoordinates(location.getAttribute("data-lat"), location.getAttribute("data-lng")), {
        id: location.getAttribute("id"),
        title: location.querySelector(".location-title").textContent.trim(),
        label: location.getAttribute("data-label") ?? undefined,
        icon: location.getAttribute("data-icon") ?? undefined,
        draggable: location.getAttribute("data-draggable") ?? undefined,
        wrapper: location,
      });

      this.addMarker(marker);
    });

    return this;
  }

  addMarker(marker) {
    if (!marker.id ?? false) {
      marker.id = this.markers.length.toString();
    }

    this.markers.push(marker);

    for (const feature of this.features) {
      try {
        feature.onMarkerAdded(marker);
      } catch (e) {
        console.error("Feature failed onMarkerAdded: " + e.toString());
      }
    }

    return marker;
  }

  updateMarker(marker) {
    for (const feature of this.features) {
      try {
        feature.onMarkerUpdated(marker);
      } catch (e) {
        console.error("Feature failed onMarkerUpdated: " + e.toString());
      }
    }
  }

  removeMarker(marker) {
    for (const feature of this.features) {
      try {
        feature.onMarkerRemove(marker);
      } catch (e) {
        console.error("Feature failed onMarkerRemove: " + e.toString());
      }
    }

    this.markers.forEach((element, index) => {
      if (element.id === marker.id) {
        this.markers.splice(Number(index), 1);
      }
    });
  }

  removeMarkers() {
    while (this.markers.length) {
      this.removeMarker(this.markers.pop());
    }
  }

  clickMarker(marker) {
    for (const feature of this.features) {
      try {
        feature.onMarkerClicked(marker);
      } catch (e) {
        console.error("Feature failed onMarkerClicked: " + e.toString());
      }
    }
  }

  getMarkerById(id) {
    for (const marker of this.markers) {
      if (marker.id ?? null === id) {
        return marker;
      }
    }
    return null;
  }

  loadShapesFromContainer() {
    let shapes = [];
    this.map.wrapper.find(".geolocation-shape").each(function (index, shapeElement) {
      let meta = shapeWrapper.find('span[typeof="GeoShape"] meta').first();
      if (meta.length === 0) {
        return;
      }

      let type = meta.attr("property").toString();

      let coordinates = [];
      meta
        .attr("content")
        .toString()
        .split(" ")
        .forEach(function (rawCoordinate) {
          let coordinate = rawCoordinate.split(",");
          if (coordinate[0].length === 0 || coordinate[1].length === 0) {
            return;
          }
          coordinates.push({
            lat: parseFloat(coordinate[0]),
            lng: parseFloat(coordinate[1]),
          });
        });

      /** @type {GeolocationShape} */
      let shape = new GeolocationShape({
        coordinates: coordinates,
        shape: type,
        shapeWrapper: shapeWrapper,
      });

      switch (type) {
        case "line":
          shape.title = shapeWrapper.find(".polyline-title").text().trim();
          break;

        case "polygon":
          shape.title = shapeWrapper.find(".polygon-title").text().trim();

          if (typeof shapeWrapper.data("fillColor") !== "undefined") {
            shape.fillColor = shapeWrapper.data("fillColor").toString();
          }
          if (typeof shapeWrapper.data("fillOpacity") !== "undefined") {
            shape.fillOpacity = parseFloat(shapeWrapper.data("fillOpacity").toString());
          }
          break;
      }

      if (typeof shapeWrapper.data("strokeColor") !== "undefined") {
        shape.strokeColor = shapeWrapper.data("strokeColor").toString();
      }
      if (typeof shapeWrapper.data("strokeWidth") !== "undefined") {
        shape.strokeWidth = parseInt(shapeWrapper.data("strokeWidth").toString());
      }
      if (typeof shapeWrapper.data("strokeOpacity") !== "undefined") {
        shape.strokeOpacity = parseFloat(shapeWrapper.data("strokeOpacity").toString());
      }

      shapes.push(shape);
    });

    return shapes;
  }
}
