import { GeolocationCoordinates } from "../../../../js/GeolocationCoordinates.js";
import { GeolocationMapBase } from "../../../../js/MapProvider/GeolocationMapBase.js";
import { GeolocationBoundaries } from "../../../../js/GeolocationBoundaries.js";
import { BaiduMapMarker } from "../BaiduMapMarker.js";

/**
 * @typedef BaiduMapSettings
 *
 * @extends GeolocationMapSettings
 *
 * @property {String} baidu_api_url
 * @property {MapOptions} baidu_settings
 */

/**
 * @property {BMap.Map} baiduMap
 * @property {BMap.Control[]} customControls
 * @property {BaiduMapSettings} settings
 */
export default class Baidu extends GeolocationMapBase {
  /**
   * @constructor
   *
   * @param {BaiduMapSettings} mapSettings
   */
  constructor(mapSettings) {
    super(mapSettings);

    this.customControls = [];

    // Set the container size.
    this.container.style.height = this.settings.baidu_settings.height;
    this.container.style.width = this.settings.baidu_settings.width;
  }

  initialize() {
    return super.initialize()
      .then(() => {
        return new Promise((resolve) => {
          Drupal.geolocation.maps.addMapProviderCallback('Baidu', resolve);
        });
      })
      .then(() => {
        return new Promise((resolve) => {
          this.baiduMap = new BMap.Map(
            this.container,
            Object.assign(this.settings.baidu_settings, {
              zoom: this.settings.baidu_settings.zoom ?? 2,
            })
          );

          resolve();
        }).then(() => {
          return new Promise((resolve) => {
            let singleClick;

            this.baiduMap.addEventListener("click", (event) => {
              singleClick = setTimeout(() => {
                for (const feature of this.features) {
                  feature.onClick(new GeolocationCoordinates(event.point.lat, event.point.lng));
                }
              }, 500);
            });

            this.baiduMap.addEventListener("dblclick", (event) => {
              clearTimeout(singleClick);
              for (const feature of this.features) {
                feature.onDoubleClick(new GeolocationCoordinates(event.point.lat, event.point.lng));
              }
            });

            this.baiduMap.addEventListener("rightclick", (event) => {
              for (const feature of this.features) {
                feature.onContextClick(new GeolocationCoordinates(event.point.lat, event.point.lng));
              }
            });

            this.baiduMap.addEventListener("moveend", () => {
              this.updatingBounds = false;

              for (const feature of this.features) {
                feature.onMapIdle();
              }
            });

            this.baiduMap.addEventListener("moveend", () => {
              let bounds = this.getBoundaries();
              if (!bounds) {
                return;
              }

              for (const feature of this.features) {
                feature.onBoundsChanged(bounds);
              }
            });

            resolve(this);
          });
        });
      });
  }

  createMarker(coordinates, settings) {
    let marker = new BaiduMapMarker(coordinates, settings, this);
    this.baiduMap.addOverlay(marker.baiduMarker);

    return marker;
  }

  addShape(shapeSettings) {
    if (typeof shapeSettings === "undefined") {
      return;
    }

    let shape;

    switch (shapeSettings.shape) {
      case "line":
        shape = new Baidu.maps.Polyline({
          path: shapeSettings.coordinates,
          strokeColor: shapeSettings.strokeColor,
          strokeOpacity: shapeSettings.strokeOpacity,
          strokeWeight: shapeSettings.strokeWidth,
        });
        break;

      case "polygon":
        shape = new Baidu.maps.Polygon({
          paths: shapeSettings.coordinates,
          strokeColor: shapeSettings.strokeColor,
          strokeOpacity: shapeSettings.strokeOpacity,
          strokeWeight: shapeSettings.strokeWidth,
          fillColor: shapeSettings.fillColor,
          fillOpacity: shapeSettings.fillOpacity,
        });
        break;
    }

    if (typeof shapeSettings.title !== "undefined" && shapeSettings.title.length) {
      /** @type BMap.InfoWindow */
      let infoWindow = new Baidu.maps.InfoWindow(shapeSettings.title);
      shape.addEventListener('onmouseover', (e) => {
        this.baiduMap.openInfoWindow(infoWindow, e.point);
      });
      shape.addEventListener("onmouseout", () => {
        this.baiduMap.closeInfoWindow();
      });
    }

    this.baiduMap.addOverlay(shape);
    super.addShape(shape);

    return shape;
  }

  removeShape(shape) {
    if (typeof shape === "undefined") {
      return;
    }
    super.removeShape(shape);
    shape.setMap(null);
  }

  getBoundaries() {
    super.getBoundaries();

    return this.normalizeBoundaries(this.baiduMap.getBounds());
  }

  getMarkerBoundaries(markers) {
    super.getMarkerBoundaries(markers);

    markers = markers || this.dataLayers.get('default').markers;
    if (!markers) {
      return null;
    }

    let bounds;

    for (const marker of markers) {
      if (!bounds) {
        bounds = new BMap.Bounds(marker.baiduMarker.getPosition(), marker.baiduMarker.getPosition());
      } else {
        bounds.extend(marker.baiduMarker.getPosition());
      }
    }

    return this.normalizeBoundaries(bounds);
  }

  setBoundaries(boundaries) {
    if (super.setBoundaries(boundaries) === false) {
      return false;
    }

    /** @type {BMap.Bounds} */
    boundaries = this.denormalizeBoundaries(boundaries);

    this.baiduMap.setViewport([boundaries.getNorthEast(), boundaries.getSouthWest()]);

    return this;
  }

  getZoom() {
    this.baiduMap.getZoom();
  }

  setZoom(zoom, defer) {
    if (!zoom) {
      zoom = this.settings.baidu_settings.zoom;
    }
    zoom = parseInt(zoom);

    this.baiduMap.setZoom(zoom);
  }

  getCenter() {
    let center = this.baiduMap.getCenter();

    return new GeolocationCoordinates(center.lat, center.lng);
  }

  setCenterByCoordinates(coordinates, accuracy) {
    super.setCenterByCoordinates(coordinates, accuracy);

    if (typeof accuracy === "undefined") {
      this.baiduMap.panTo(new BMap.Point(coordinates.lng, coordinates.lat));
      return;
    }

    let circle = this.addAccuracyIndicatorCircle(coordinates, accuracy);

    // Set the zoom level to the accuracy circle's size.
    this.setBoundaries(this.normalizeBoundaries(circle.getBounds()));

    // Fade circle away.
    setInterval(() => {
      let fillOpacity = circle.getFillOpacity();
      fillOpacity -= 0.01;

      let strokeOpacity = circle.getStrokeOpacity();
      strokeOpacity -= 0.02;

      if (strokeOpacity > 0 && fillOpacity > 0) {
        circle.setFillOpacity(fillOpacity);
        circle.setStrokeOpacity(strokeOpacity);
      } else {
        this.baiduMap.removeOverlay(circle);
      }
    }, 200);
  }

  normalizeBoundaries(boundaries) {
    if (boundaries instanceof GeolocationBoundaries) {
      return boundaries;
    }

    if (boundaries instanceof BMap.Bounds) {
      if (boundaries.isEmpty()) {
        return null;
      }
      return new GeolocationBoundaries({
        north: boundaries.getNorthEast().lat,
        east: boundaries.getNorthEast().lng,
        south: boundaries.getSouthWest().lat,
        west: boundaries.getSouthWest().lng,
      });
    }

    return false;
  }

  denormalizeBoundaries(boundaries) {
    if (boundaries instanceof BMap.Bounds) {
      return boundaries;
    }

    if (boundaries instanceof GeolocationBoundaries) {
      return new BMap.Bounds(new BMap.Point(boundaries.east, boundaries.north), new BMap.Point(boundaries.west, boundaries.south));
    }

    return false;
  }

  addControl(element) {
    let control = new BMap.Control();
    control.setAnchor(element.getAttribute("data-map-control-position") ?? BMAP_ANCHOR_TOP_LEFT);
    control.initialize = (map) => {
      map.getContainer().appendChild(element);
    };

    this.baiduMap.addControl(control);
  }

  removeControls() {
    for (const control of this.customControls) {
      this.baiduMap.removeControl(control);
    }
  }

  addAccuracyIndicatorCircle(coordinates, accuracy) {
    const circle = new BMap.Circle(new BMap.Point(coordinates.lng, coordinates.lat), accuracy, {
      fillColor: "#4285F4",
      fillOpacity: 0.15,
      strokeColor: "#4285F4",
      strokeOpacity: 0.3,
      strokeWeight: 1,
      enableClicking: false,
    });

    this.baiduMap.addOverlay(circle);

    return circle;
  }
}
