// noinspection TypeScriptUMDGlobal

import { GeolocationCoordinates } from "../../../../js/GeolocationCoordinates.js";
import { GeolocationMapBase } from "../../../../js/MapProvider/GeolocationMapBase.js";
import { GeolocationBoundaries } from "../../../../js/GeolocationBoundaries.js";
import { LeafletMapMarker } from "../LeafletMapMarker.js";

/**
 * @typedef LeafletSettings
 *
 * @extends GeolocationMapSettings
 *
 * @property {L.MapOptions} leaflet_settings
 */

/**
 * @property {LeafletSettings} settings
 * @property {L.LayerGroup} markerLayer
 * @property {L.Control[]} controls
 */
export default class Leaflet extends GeolocationMapBase {
  /**
   * @constructor
   *
   * @param {LeafletSettings} mapSettings
   */
  constructor(mapSettings) {
    super(mapSettings);

    // Set the container size.
    this.container.style.height = this.settings.leaflet_settings.height ?? "400px";
    this.container.style.width = this.settings.leaflet_settings.width ?? "100%";
  }

  initialize() {
    return super.initialize()
    .then(() => {
      return new Promise((resolve) => {
        let mapOptions = Object.assign(this.settings.leaflet_settings, {
          attributionControl: this.settings.leaflet_settings.attributionControl ?? false,
          bounceAtZoomLimits: this.settings.leaflet_settings.bounceAtZoomLimits ?? true,
          center: [this.settings.lat, this.settings.lng],
          crs: L.CRS[this.settings.leaflet_settings.crs ?? "EPSG3857"],
          doubleClickZoom: this.settings.leaflet_settings.doubleClickZoom ?? true,
          zoom: this.settings.leaflet_settings.zoom ?? 10,
          minZoom: this.settings.leaflet_settings.minZoom ?? 0,
          maxZoom: this.settings.leaflet_settings.maxZoom ?? 20,
          zoomControl: this.settings.leaflet_settings.zoomControl ?? false,
          preferCanvas: this.settings.leaflet_settings.preferCanvas ?? false,
          zoomSnap: this.settings.leaflet_settings.zoomSnap ?? 1,
          zoomDelta: this.settings.leaflet_settings.zoomDelta ?? 1,
          trackResize: this.settings.leaflet_settings.trackResize ?? true,
          boxZoom: this.settings.leaflet_settings.boxZoom ?? true,
          dragging: this.settings.leaflet_settings.dragging ?? true,
          zoomAnimation: this.settings.leaflet_settings.zoomAnimation ?? true,
          zoomAnimationThreshold: this.settings.leaflet_settings.zoomAnimationThreshold ?? 4,
          fadeAnimation: this.settings.leaflet_settings.fadeAnimation ?? true,
          markerZoomAnimation: this.settings.leaflet_settings.markerZoomAnimation ?? true,
          inertia: this.settings.leaflet_settings.inertia ?? false,
          inertiaDeceleration: this.settings.leaflet_settings.inertiaDeceleration ?? 3000,
          easeLinearity: this.settings.leaflet_settings.easeLinearity ?? 0.2,
          worldCopyJump: this.settings.leaflet_settings.worldCopyJump ?? false,
          maxBoundsViscosity: this.settings.leaflet_settings.maxBoundsViscosity ?? 0.0,
          keyboard: this.settings.leaflet_settings.keyboard ?? true,
          keyboardPanDelta: this.settings.leaflet_settings.keyboardPanDelta ?? 80,
          scrollWheelZoom: this.settings.leaflet_settings.scrollWheelZoom ?? true,
          wheelDebounceTime: this.settings.leaflet_settings.wheelDebounceTime ?? 40,
          wheelPxPerZoomLevel: this.settings.leaflet_settings.wheelPxPerZoomLevel ?? 60,
          tap: this.settings.leaflet_settings.tap ?? true,
          tapTolerance: this.settings.leaflet_settings.tapTolerance ?? 15,
          touchZoom: this.settings.leaflet_settings.touchZoom ?? true,
        });

        this.leafletMap = L.map(this.container, mapOptions);

        this.markerLayer = L.layerGroup().addTo(this.leafletMap);
        this.tileLayer = L.tileLayer("https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png").addTo(this.leafletMap);

        resolve();
      })
      .then(() => {
        return new Promise((resolve) => {
          let singleClick;

          this.leafletMap.on(
            "click",
            /** @param {L.LeafletMouseEvent} e */ (e) => {
              singleClick = setTimeout(() => {
                for (const feature of this.features) {
                  feature.onClick(new GeolocationCoordinates(e.latlng.lat, e.latlng.lng));
                }
              }, 500);
            }
          );

          this.leafletMap.on(
            "dblclick",
            /** @param {L.LeafletMouseEvent} e */ (e) => {
              clearTimeout(singleClick);
              for (const feature of this.features) {
                feature.onDoubleClick(new GeolocationCoordinates(e.latlng.lat, e.latlng.lng));
              }
            }
          );

          this.leafletMap.on(
            "contextmenu",
            /** @param {L.LeafletMouseEvent} e */ (e) => {
              for (const feature of this.features) {
                feature.onContextClick(new GeolocationCoordinates(e.latlng.lat, e.latlng.lng));
              }
            }
          );

          this.leafletMap.on("moveend", () => {
            let bounds = this.getBoundaries();

            for (const feature of this.features) {
              feature.onMapIdle();
              if (!bounds) {
                return;
              }
              feature.onBoundsChanged(this.normalizeBoundaries(bounds));
            }
          });

          resolve(this);
        });
      });
    });
  }

  createMarker(coordinates, settings) {
    return new LeafletMapMarker(coordinates, settings, this);
  }

  addShape(shapeSettings) {
    if (typeof shapeSettings === "undefined") {
      return;
    }

    let shape;

    switch (shapeSettings.shape) {
      case "line":
        shape = L.polyline(shapeSettings.coordinates, {
          color: shapeSettings.strokeColor,
          opacity: shapeSettings.strokeOpacity,
          weight: shapeSettings.strokeWidth,
        });
        if (shapeSettings.title) {
          shape.bindTooltip(shapeSettings.title);
        }
        break;

      case "polygon":
        shape = L.polygon(shapeSettings.coordinates, {
          color: shapeSettings.strokeColor,
          opacity: shapeSettings.strokeOpacity,
          weight: shapeSettings.strokeWidth,
          fillColor: shapeSettings.fillColor,
          fillOpacity: shapeSettings.fillOpacity,
        });
        if (shapeSettings.title) {
          shape.bindTooltip(shapeSettings.title);
        }
        break;
    }

    shape.addTo(this.leafletMap);
    super.addShape(shape);

    return shape;
  }

  removeShape(shape) {
    if (typeof shape === "undefined") {
      return;
    }
    super.removeShape(shape);
    shape.remove();
  }

  getMarkerBoundaries(markers) {
    super.getMarkerBoundaries();

    markers = markers || this.dataLayers.get('default').markers;
    if (!markers) {
      return null;
    }

    let leafletMarkers = [];
    for (const marker of markers) {
      leafletMarkers.push(marker.leafletMarker);
    }

    let group = new L.featureGroup(leafletMarkers);

    return this.normalizeBoundaries(group.getBounds());
  }

  getBoundaries() {
    return this.normalizeBoundaries(this.leafletMap.getBounds());
  }

  setBoundaries(boundaries) {
    if (super.setBoundaries(boundaries) === false) {
      return false;
    }

    this.leafletMap.fitBounds(this.denormalizeBoundaries(boundaries) ?? false);
  }

  getZoom() {
    return new Promise((resolve) => {
      resolve(this.leafletMap.getZoom());
    });
  }

  setZoom(zoom, defer) {
    if (!zoom) {
      zoom = this.settings.leaflet_settings.zoom;
    }
    this.leafletMap.setZoom(parseInt(zoom));
  }

  getCenter() {
    let center = this.leafletMap.getCenter();

    return new GeolocationCoordinates(center.lat, center.lng);
  }

  setCenterByCoordinates(coordinates, accuracy) {
    super.setCenterByCoordinates(coordinates, accuracy);

    if (typeof accuracy === "undefined") {
      this.leafletMap.panTo(coordinates);
      return;
    }

    // TODO: Is this copied from Google? Does it work?

    let circle = this.addAccuracyIndicatorCircle(coordinates, accuracy);

    this.leafletMap.fitBounds(circle.getBounds());

    setInterval(() => {
      let fillOpacity = circle.options.fillOpacity;
      fillOpacity -= 0.03;

      let opacity = circle.options.opacity;
      opacity -= 0.06;

      if (opacity > 0 && fillOpacity > 0) {
        circle.setStyle({
          fillOpacity: fillOpacity,
          stroke: opacity,
        });
      } else {
        circle.remove();
      }
    }, 300);
  }

  normalizeBoundaries(boundaries) {
    if (boundaries instanceof GeolocationBoundaries) {
      return boundaries;
    }

    if (boundaries instanceof L.LatLngBounds) {
      return new GeolocationBoundaries({
        north: boundaries.getNorth(),
        east: boundaries.getEast(),
        south: boundaries.getSouth(),
        west: boundaries.getWest(),
      });
    }

    return false;
  }

  /**
   * @param {GeolocationBoundaries} boundaries
   *
   * @returns {L.LatLngBoundsExpression|false}
   */
  denormalizeBoundaries(boundaries) {
    if (typeof boundaries === "undefined") {
      return false;
    }

    if (boundaries instanceof L.LatLngBounds) {
      return boundaries;
    }

    if (boundaries instanceof GeolocationBoundaries) {
      return L.latLngBounds([
        [boundaries.south, boundaries.west],
        [boundaries.north, boundaries.east],
      ]);
    }

    return false;
  }

  addControl(element) {
    this.controls = this.controls || [];
    let controlElement = new (L.Control.extend({
      options: {
        position: typeof element.dataset.controlPosition === "undefined" ? "topleft" : element.dataset.controlPosition,
      },
      onAdd: () => {
        element.style.display = "block";
        L.DomEvent.disableClickPropagation(element);
        return element;
      },
    }))();
    controlElement.addTo(this.leafletMap);
    this.controls.push(controlElement);
  }

  removeControls() {
    this.controls = this.controls || [];
    for (const control of this.controls) {
      this.leafletMap.removeControl(control);
    }
  }

  addAccuracyIndicatorCircle(location, accuracy) {
    return L.circle(location, accuracy, {
      interactive: false,
      color: "#4285F4",
      opacity: 0.3,
      fillColor: "#4285F4",
      fillOpacity: 0.15,
    }).addTo(this.leafletMap);
  }
}
