import { GeolocationMapBase } from "../../../../js/MapProvider/GeolocationMapBase.js";
import { GeolocationBoundaries } from "../../../../js/GeolocationBoundaries.js";
import { GeolocationCoordinates } from "../../../../js/GeolocationCoordinates.js";
import { YandexMapMarker } from "../YandexMapMarker.js";

/**
 * @typedef YandexMapSettings
 *
 * @extends GeolocationMapSettings
 *
 * @property {MapOptions} yandex_settings
 * @property {Number} yandex_settings.zoom
 * @property {Number} yandex_settings.max_zoom
 * @property {Number} yandex_settings.min_zoom
 */

/**
 * @property {ymaps.Map} yandexMap
 * @property {YandexMapSettings} settings
 */
export default class Yandex extends GeolocationMapBase {
  /**
   * @constructor
   *
   * @param {YandexMapSettings} mapSettings
   */
  constructor(mapSettings) {
    super(mapSettings);

    this.customControls = [];

    // Set the container size.
    this.container.style.height = this.settings.yandex_settings.height;
    this.container.style.width = this.settings.yandex_settings.width;
  }

  initialize() {
    return super.initialize()
      .then(() => {
        return new Promise((resolve) => {
          ymaps.ready(() => {
            resolve();
          });
        });
      })
      .then(() => {
        return new Promise((resolve) => {
          this.yandexMap = new ymaps.Map(
            this.wrapper,
            {
              center: [0, 0],
              zoom: this.settings.yandex_settings.zoom,
              controls: [],
            },
            {
              maxZoom: this.settings.yandex_settings.max_zoom ?? 20,
              minZoom: this.settings.yandex_settings.min_zoom ?? 0,
            }
          );

          resolve();
        }).then(() => {
          return new Promise((resolve) => {
            let singleClick;

            this.yandexMap.events.add("click", (e) => {
              singleClick = setTimeout(() => {
                for (const feature of this.features) {
                  feature.onClick(this.normalizeCoordinates(e.get("coords")));
                }
              }, 500);
            });

            this.yandexMap.events.add("dblclick", (e) => {
              clearTimeout(singleClick);
              for (const feature of this.features) {
                feature.onDoubleClick(this.normalizeCoordinates(e.get("coords")));
              }
            });

            this.yandexMap.events.add("contextmenu", (e) => {
              for (const feature of this.features) {
                feature.onContextClick(this.normalizeCoordinates(e.get("coords")));
              }
            });

            this.yandexMap.events.add("boundschange", () => {
              this.updatingBounds = false;

              for (const feature of this.features) {
                feature.onMapIdle();
              }
            });

            this.yandexMap.events.add("boundschange", () => {
              let bounds = this.yandexMap.getBounds();
              if (!bounds) {
                return;
              }

              for (const feature of this.features) {
                feature.onBoundsChanged(this.normalizeBoundaries(bounds));
              }
            });

            resolve(this);
          });
        });
      })
      .then((map) => {
        class CustomControlClass extends ymaps.collection.Item {
          constructor(element, options = {}) {
            super(options);

            this.element;
          }

          onAddToMap(map) {
            super.onAddToMap(map);

            this.getParent()
              .getChildElement(this)
              .then((parent) => {
                parent.append(this.element);
              });
          }
        }

        return map;
      });
  }

  createMarker(coordinates, settings) {
    let marker = new YandexMapMarker(coordinates, settings, this);
    this.yandexMap.geoObjects.add(marker.yandexMarker);

    return marker;
  }

  addShape(shapeSettings) {
    if (typeof shapeSettings === "undefined") {
      return;
    }

    let shape;

    switch (shapeSettings.shape) {
      case "line":
        shape = new Yandex.maps.Polyline({
          path: shapeSettings.coordinates,
          strokeColor: shapeSettings.strokeColor,
          strokeOpacity: shapeSettings.strokeOpacity,
          strokeWeight: shapeSettings.strokeWidth,
        });
        break;

      case "polygon":
        shape = new Yandex.maps.Polygon({
          paths: shapeSettings.coordinates,
          strokeColor: shapeSettings.strokeColor,
          strokeOpacity: shapeSettings.strokeOpacity,
          strokeWeight: shapeSettings.strokeWidth,
          fillColor: shapeSettings.fillColor,
          fillOpacity: shapeSettings.fillOpacity,
        });
        break;
    }

    if (typeof shapeSettings.title !== "undefined" && shapeSettings.title.length) {
      let infoWindow = new Yandex.maps.InfoWindow();
      let that = this;
      Yandex.maps.event.addEventListener(shape, "mouseover", function (e) {
        infoWindow.setPosition(e.point);
        infoWindow.setContent(shapeSettings.title);
        infoWindow.open(that.YandexMap);
      });
      Yandex.maps.event.addEventListener(shape, "mouseout", function () {
        infoWindow.close();
      });
    }

    shape.setMap(this.yandexMap);
    super.addShape(shape);

    return shape;
  }

  removeShape(shape) {
    if (typeof shape === "undefined") {
      return;
    }
    super.removeShape(shape);
    shape.setMap(null);
  }

  getBoundaries() {
    super.getBoundaries();

    return this.normalizeBoundaries(this.yandexMap.getBounds());
  }

  getMarkerBoundaries(markers) {
    super.getMarkerBoundaries(markers);

    markers = markers || this.dataLayers.get('default').markers;
    if (!markers) {
      return null;
    }

    return this.normalizeBoundaries(this.yandexMap.geoObjects.getBounds());
  }

  setBoundaries(boundaries) {
    if (super.setBoundaries(boundaries) === false) {
      return false;
    }

    return this.yandexMap.setBounds(this.denormalizeBoundaries(boundaries));
  }

  getZoom() {
    this.yandexMap.getZoom();
  }

  setZoom(zoom, defer) {
    if (!zoom) {
      zoom = this.settings.yandex_settings.zoom;
    }
    zoom = parseInt(zoom);

    return this.yandexMap.setZoom(zoom);
  }

  getCenter() {
    return this.normalizeCoordinates(this.yandexMap.getCenter());
  }

  setCenterByCoordinates(coordinates, accuracy) {
    super.setCenterByCoordinates(coordinates, accuracy);

    this.yandexMap.panTo(this.denormalizeCoordinates(coordinates));
  }

  normalizeCoordinates(coordinates) {
    if (coordinates instanceof GeolocationCoordinates) {
      return coordinates;
    }

    return new GeolocationCoordinates(coordinates[0], coordinates[1]);
  }

  denormalizeCoordinates(coordinates) {
    if (!(coordinates instanceof GeolocationCoordinates)) {
      return coordinates;
    }

    return [coordinates.lat, coordinates.lng];
  }

  normalizeBoundaries(boundaries) {
    if (boundaries instanceof GeolocationBoundaries) {
      return boundaries;
    }

    return new GeolocationBoundaries({
      north: boundaries[1][1],
      east: boundaries[1][0],
      south: boundaries[0][1],
      west: boundaries[0][0],
    });
  }

  denormalizeBoundaries(boundaries) {
    if (boundaries instanceof GeolocationBoundaries) {
      return [
        [boundaries.west, boundaries.south],
        [boundaries.east, boundaries.north],
      ];
    }

    return false;
  }

  addControl(element) {
    const control = new CustomControlClass(element);
    this.customControls.push(control);
    this.yandexMap.controls.add(control, {
      float: "none",
      position: {
        top: 10,
        left: 10,
      },
    });
  }

  removeControls() {
    for (const control of this.customControls) {
      this.yandexMap.controls.remove(control);
    }
  }
}
